
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity vgaController_tb is
    --  Port ( );
end vgaController_tb;

architecture Behavioral of vgaController_tb is
    -- Deklaracija modula UUT
    component vgaController is
        Port (
            CLK100MHZ: in  std_logic;
            BTNC:      in  std_logic;
            VGA_HS:    out std_logic;
            VGA_VS:    out std_logic;
            VGA_R:     out std_logic_vector (3 downto 0);
            VGA_G:     out std_logic_vector (3 downto 0);
            VGA_B:     out std_logic_vector (3 downto 0)
        );
    end component;

    -- Vhodni signali: povezujejo generator stimulusov z vhodi modula
    signal clk, rst: std_logic := '0';

    -- Izhodni signali: povezujejo izhode modula s osciloskopom (wave forms)
    signal hsync: std_logic := '0';
    signal vsync: std_logic := '0';
    signal vgaRed, vgaGreen, vgaBlue:  std_logic_vector(3 downto 0);

    -- Konstante 
    constant clock_period: time := 10 ns;
begin

    -- Instanciacija UUT 
    UUT: vgaController
        port map (
            BTNC              => rst,
            CLK100MHZ         => clk,
            VGA_HS            => hsync,
            VGA_VS            => vsync,
            VGA_B(3 downto 0) => vgaBlue(3 downto 0),
            VGA_G(3 downto 0) => vgaGreen(3 downto 0),
            VGA_R(3 downto 0) => vgaRed(3 downto 0)
        );

    -- Generator stimulusov 
    -- 1. Ura 
    clk_stimuli: process
    begin
        clk <= not clk;
        wait for clock_period/2;
    end process;

    -- 2. Ostali stimulusi
    oth_stimuli: process
    begin
        -- dolgi reset
        rst <= '1';
        wait for clock_period *3;
        rst <= '0';
        wait;  -- cakaj v neskoncnost
    end process;
end Behavioral;
